<?php
if ( ! defined( 'ABSPATH' ) ) exit;

class IRPP_Manager {

    public function __construct() {

    }

    public function getRelatedPostsIds($args) {
        global $irpp;

        $ids=array();
        if(IRPP_DISABLE_RELATED || !$irpp->Options->isActive()) {
            return $ids;
        }

        $defaults=array('postId'=>0
            , 'posts'=>array(), 'cats'=>array(), 'tags'=>array()
            , 'count'=>10
            , 'shuffle'=>FALSE);
        $args=$irpp->Utils->parseArgs($args, $defaults);

        $args['posts']=$irpp->Utils->toCommaArray($args['posts']);
        $args['cats']=$irpp->Utils->toCommaArray($args['cats']);
        $args['tags']=$irpp->Utils->toCommaArray($args['tags']);
        $args['postId']=intval($args['postId']);
        if($args['postId']>0) {
            $options=array('orderby' => 'name', 'order' => 'ASC', 'fields' => 'ids');
            $args['tags']=wp_get_post_tags($args['postId'], $options);
            $args['cats']=wp_get_post_categories($args['postId']);
        }

        $engineSearch = $irpp->Options->getEngineSearch();

        if ($engineSearch == IRPP_ENGINE_SEARCH_CATEGORIES_SUBCATEGORIES
            || $engineSearch == IRPP_ENGINE_SEARCH_CATEGORIES_SUBCATEGORIES_TAGS) {
            foreach ($args['cats'] as $c) {
                $termChildren = get_term_children($c, 'category');
                foreach ($termChildren as $k=>$v) {
                    $args['cats'][] = $v;
                }

                $parentID = wp_get_term_taxonomy_parent_id($c, 'category');
                if ($parentID != false) {
                    $args['cats'][] = $parentID;
                }
            }
        }

        // handle excluded categories and tags
        $excludeCats = $irpp->Options->getExcludedCategoriesAsId();
        $excludeTags = $irpp->Options->getExcludedTagsAsId();
        if (!empty($excludeCats) || !empty($excludeTags)) {
            foreach ($excludeCats as $e) {
                foreach ($args['cats'] as $x=>$y) {
                    if ($y == $e) {
                        unset($args['cats'][$x]);
                    }
                }
            }
            foreach ($excludeTags as $t) {
                foreach ($args['tags'] as $x=>$y) {
                    if ($y == $t) {
                        unset($args['tags'][$x]);
                    }
                }
            }
        }
        if (count($args['posts']) > 0) {
            $ids = $args['posts'];
        } else {
            switch ($engineSearch) {
                case IRPP_ENGINE_SEARCH_CATEGORIES:
                case IRPP_ENGINE_SEARCH_CATEGORIES_SUBCATEGORIES:
                    $ids = $this->queryRelatedPostsIds($args, 'category__in', $args['cats'], $excludeCats, $excludeTags);
                    break;
                case IRPP_ENGINE_SEARCH_TAGS:
                    $ids = $this->queryRelatedPostsIds($args, 'tag__in', $args['tags'], $excludeCats, $excludeTags);
                    break;
                case IRPP_ENGINE_SEARCH_CATEGORIES_TAGS:
                case IRPP_ENGINE_SEARCH_CATEGORIES_SUBCATEGORIES_TAGS:
                    //if we execute this query with the 2 parameters together category__in
                    //and tag__in WP do an AND query and not an OR query as we need
                    $cats = $this->queryRelatedPostsIds($args, 'category__in', $args['cats'], $excludeCats, $excludeTags);
                    $tags = $this->queryRelatedPostsIds($args, 'tag__in', $args['tags'], $excludeCats, $excludeTags);
                    $ids = array_merge($cats, $tags);
                    $ids = array_unique($ids);
                    break;
            }
        }

        if($args['shuffle']) {
            shuffle($ids);
        }
        return $ids;
    }

    private function queryRelatedPostsIds($args, $optionKey, $optionArray, $excludeCats = null, $excludeTags = null)
    {
        global $irpp;

        $ids=array();
        if(!is_array($optionArray) || count($optionArray)==0) {
            return $ids;
        }

        $postTypes = $irpp->Utils->getSelectedPostTypes();

        $post=$irpp->Options->getPostShown();
        $options=array(
            'post_type' => $postTypes //'post_type'=>$post->post_type
            //, 'nopaging'=>TRUE
            , 'posts_per_page'=>10
            , 'post_status'=>'publish'
            , 'post__not_in'=>array($post->ID)
            , 'orderby'=>'rand'
        );
        $days=$irpp->Options->getRewritePostsDays();
        if($days>0) {
            $options['date_query'] = array(
                'column' => 'post_date'
                , 'after' => '- '.$days.' days'
            );
        }
        $options[$optionKey]=$optionArray;

        if (!empty($excludeCats)) {
            $options['category__not_in'] = $excludeCats;
         }

         if (!empty($excludeTags)) {
            $options['tag__not_in'] = $excludeTags;
         }

        $q=new WP_Query();
        $irpp->Log->debug('RELATED POSTS QUERY=%s', $options);
        $posts=$q->query($options);

        $irpp->Log->debug('RELATED POSTS RESULT COUNT=%s', count($posts));
        foreach($posts as $p) {
            if($args['postId']<=0 || $args['postId']!=$p->ID) {
                $ids[]=$p->ID;
            }
        }
        return $ids;
    }
}